@@ -1,3 +1,4 @@
+from copy import copy
 from functools import lru_cache
 import random
 import re
@@ -488,7 +489,7 @@ class DatatableAJAXView(BaseDatatableView):
 
         :rtype: QuerySet
         """
-        attr_name = 'search_{}_column'.format(column)
+        attr_name = 'filter_{}_column'.format(column)
         if hasattr(self, attr_name):
             qs = getattr(self, attr_name)(qs, column, value)
         elif column in self.searchable_columns():
@@ -511,8 +512,12 @@ class DatatableAJAXView(BaseDatatableView):
         """
         query = None
         for column in self.searchable_columns():
-            kwarg_name = '{}__icontains'.format(column)
-            q = Q(**{kwarg_name: value})
+            attr_name = 'search_{}_column'.format(column)
+            if hasattr(self, attr_name):
+                q = getattr(self, attr_name)(column, value)
+            else:
+                kwarg_name = '{}__icontains'.format(column)
+                q = Q(**{kwarg_name: value})
             query = query | q if query else q
         return query
 
@@ -634,11 +639,11 @@ class DataTableStyler:
 class DataTableForm:
 
     def __init__(self, table):
-        if table.form_actions:
+        if table.has_form_actions():
             self.is_visible = True
         else:
             self.is_visible = False
-        self.actions = table.form_actions
+        self.actions = table.get_form_actions()
         self.url = table.form_url
 
 
@@ -652,6 +657,7 @@ class DataTable(Widget, WidgetInitKwargsMixin, DatatableAJAXView):
     default_action_button_label = 'View'
     default_action_button_color_class = 'secondary'
     sort_ascending = True
+    action_button_size = 'normal'
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -673,6 +679,7 @@ class DataTable(Widget, WidgetInitKwargsMixin, DatatableAJAXView):
         self.column_filters = {}
         self.column_styles = []
         self.data = kwargs.get('data', [])
+        self._form_actions = copy(self.form_actions)
         if self.form_actions:
             self.column_fields['checkbox'] = DataTableColumn(
                 field='checkbox',
@@ -681,6 +688,20 @@ class DataTable(Widget, WidgetInitKwargsMixin, DatatableAJAXView):
                 sortable=False
             )
         self.sort_ascending = kwargs.get('sort_ascending', self.sort_ascending)
+        self.action_button_size = kwargs.get('action_button_size', self.action_button_size)
+        if not self.action_button_size == 'normal':
+            self.action_button_size_class = f"btn-{self.action_button_size}"
+        else:
+            self.action_button_size_class = ''
+
+    def has_form_actions(self):
+        return not self._form_actions == None
+
+    def get_form_actions(self):
+        return self._form_actions
+
+    def add_form_action(self, action):
+        self._form_actions.append(action)
 
     def get_order_columnsx(self):
         cols = []
@@ -705,6 +726,9 @@ class DataTable(Widget, WidgetInitKwargsMixin, DatatableAJAXView):
     def add_filter(self, field, filter):
         self.column_filters[field] = filter
 
+    def remove_filter(self, field):
+        del self.column_filters[field]
+
     def add_styler(self, styler):
         styler.test_index = list(self.column_fields.keys()).index(styler.test_cell)
         if styler.target_cell:
@@ -712,7 +736,8 @@ class DataTable(Widget, WidgetInitKwargsMixin, DatatableAJAXView):
         self.column_styles.append(styler)
 
     def build_context(self):
-        return {'rows': self.prepare_results(self.data)}
+        return {'rows': self.data}
+        # return {'rows': self.prepare_results(self.data)}
 
     def get_content(self, **kwargs):
         if self.actions:
@@ -737,6 +762,12 @@ class DataTable(Widget, WidgetInitKwargsMixin, DatatableAJAXView):
             context = {"async": True}
         html_template = template.loader.get_template(template_file)
         context['header'] = self.column_fields
+        context['has_form_actions'] = self.has_form_actions()
+        if self.actions:
+            context['has_actions'] = True
+            context['action_column'] = len(self.column_fields) - 1
+        else:
+            context['has_actions'] = False        
         context['filters'] = filters
         context['stylers'] = self.column_styles
         context['has_filters'] = has_filters
@@ -776,17 +807,23 @@ class DataTable(Widget, WidgetInitKwargsMixin, DatatableAJAXView):
             url = "javascript:void(0)"
         return self.get_action_button_with_url(row, label, url, method, color_class, attr, js_function_name)
 
+    def get_action_button_url_extra_attributes(self, row):
+        return ""
+
     def get_action_button_with_url(self, row, label, url, method='get', color_class='secondary', attr='id',
                                    js_function_name=None):
+        url_extra = self.get_action_button_url_extra_attributes(row)
+        if url_extra:
+            url = f"{url}&{url_extra}"                                   
         if method == 'get':
             if js_function_name:
                 link_extra = f"onclick='{js_function_name}({row.id});'"
             else:
                 link_extra = ""
-            return f"<a href='{url}' class='btn btn-{color_class} btn-smx me-2' {link_extra}>{label}</a>"
+            return f"<a href='{url}' class='btn btn-{color_class} {self.action_button_size_class} me-2' {link_extra}>{label}</a>"
         token_input = f'<input type="hidden" name="csrfmiddlewaretoken" value="{self.csrf_token}">'
         id_input = f'<input type="hidden" name="{attr}" value="{row.id}">'
-        button = f'<input type=submit value="{label}" class="btn btn-{color_class} btn-smx me-2">'
+        button = f'<input type=submit value="{label}" class="btn btn-{color_class} {self.action_button_size_class} me-2">'
         form = f"<form class='form form-inline' action={url} method='post'>{token_input}{id_input}{button}</form>"
         return form
 
@@ -794,7 +831,7 @@ class DataTable(Widget, WidgetInitKwargsMixin, DatatableAJAXView):
         return ''
 
     def render_actions_column(self, row, column):
-        response = "<div class='d-flex flex-row'>"
+        response = "<div class='d-flex flex-row justify-content-end'>"
         if hasattr(row, 'get_absolute_url'):
             url = row.get_absolute_url()
             view_button = self.get_action_button_with_url(row, self.default_action_button_label, url, color_class=self.default_action_button_color_class)
@@ -923,10 +960,11 @@ class BasicModelTable(DataTable):
             kwargs = {}
             if field_name in self.verbose_names:
                 kwargs['verbose_name'] = self.verbose_names[field_name]
-            elif verbose_name == field.verbose_name:
-                kwargs['verbose_name'] = verbose_name.capitalize()
-            else:
-                kwargs['verbose_name'] = field.verbose_name
+            elif hasattr(field, 'verbose_name'):
+                if verbose_name == field.verbose_name:
+                    kwargs['verbose_name'] = verbose_name.capitalize()
+                else:
+                    kwargs['verbose_name'] = field.verbose_name
             self.set_standard_column_attributes(field_name, kwargs)
             self.add_column(field_name, **kwargs)
         else:
@@ -964,13 +1002,17 @@ class BasicModelTable(DataTable):
         datetime_format = "%m/%d/%Y %H:%M"
         if hasattr(settings, 'WILDEWIDGETS_DATETIME_FORMAT'):
             datetime_format = settings.WILDEWIDGETS_DATETIME_FORMAT
-        return value.strftime(datetime_format)
+        if value:
+            return value.strftime(datetime_format)
+        return ""
 
     def render_date_type_column(self, value):
         date_format = "%m/%d/%Y"
         if hasattr(settings, 'WILDEWIDGETS_DATE_FORMAT'):
             date_format = settings.WILDEWIDGETS_DATE_FORMAT
-        return value.strftime(date_format)
+        if value:
+            return value.strftime(date_format)
+        return ""
 
     def render_column(self, row, column):
         value = super().render_column(row, column)
